@@ -339,17 +339,17 @@ get_adjusted_glyphs_list (ThaiFontInfo *font_info,
         if (is_char_type (cluster[0], BelowVowel|BelowDiac|AboveVowel|AboveDiac|Tone))
 	  {
 	    if (font_info->font_set == THAI_FONT_TIS)
-	      glyph_lists[0] = (*font_info->make_glyph) (font_info, 0x20);
+	      glyph_lists[0] = thai_make_glyph (font_info, 0x20);
 	    else
-	      glyph_lists[0] = (*font_info->make_glyph) (font_info, 0x7F);
+	      glyph_lists[0] = thai_make_glyph (font_info, 0x7F);
             glyph_lists[1] =
-		(*font_info->make_glyph) (font_info, ucs2tis (cluster[0]));
+		thai_make_glyph (font_info, ucs2tis (cluster[0]));
 	    return 2;
           }
 	else
 	  {
 	    glyph_lists[0] =
-		(*font_info->make_glyph) (font_info, ucs2tis (cluster[0]));
+		thai_make_glyph (font_info, ucs2tis (cluster[0]));
             return 1;
           }
         break;
@@ -359,36 +359,36 @@ get_adjusted_glyphs_list (ThaiFontInfo *font_info,
             is_char_type (cluster[1], SaraAm))
 	  {
             glyph_lists[0] =
-		(*font_info->make_glyph) (font_info, ucs2tis (cluster[0]));
-            glyph_lists[1] = (*font_info->make_glyph) (font_info, 0xED);
-            glyph_lists[2] = (*font_info->make_glyph) (font_info, 0xD2);
+		thai_make_glyph (font_info, ucs2tis (cluster[0]));
+            glyph_lists[1] = thai_make_glyph (font_info, 0xED);
+            glyph_lists[2] = thai_make_glyph (font_info, 0xD2);
             return 3;
           }
 	else if (is_char_type (cluster[0], UpTailCons) &&
           	 is_char_type (cluster[1], SaraAm))
 	  {
 	    glyph_lists[0] =
-		(*font_info->make_glyph) (font_info, ucs2tis (cluster[0]));
-            glyph_lists[1] = (*font_info->make_glyph) (font_info,
+		thai_make_glyph (font_info, ucs2tis (cluster[0]));
+            glyph_lists[1] = thai_make_glyph (font_info,
 					shiftleft_tone_ad (0xED, shaping_table));
-            glyph_lists[2] = (*font_info->make_glyph) (font_info, 0xD2);
+            glyph_lists[2] = thai_make_glyph (font_info, 0xD2);
             return 3;
           }
 	else if (is_char_type (cluster[0], NoTailCons|BotTailCons|SpltTailCons) &&
           	 is_char_type (cluster[1], AboveVowel))
 	  {
 	    glyph_lists[0] =
-		(*font_info->make_glyph) (font_info, ucs2tis (cluster[0]));
+		thai_make_glyph (font_info, ucs2tis (cluster[0]));
 	    glyph_lists[1] =
-		(*font_info->make_glyph) (font_info, ucs2tis (cluster[1]));
+		thai_make_glyph (font_info, ucs2tis (cluster[1]));
 	    return 2;
           }
 	else if (is_char_type (cluster[0], NoTailCons|BotTailCons|SpltTailCons) &&
           	 is_char_type (cluster[1], AboveDiac|Tone))
 	  {
 	    glyph_lists[0] =
-		(*font_info->make_glyph) (font_info, ucs2tis (cluster[0]));
-	    glyph_lists[1] = (*font_info->make_glyph) (font_info,
+		thai_make_glyph (font_info, ucs2tis (cluster[0]));
+	    glyph_lists[1] = thai_make_glyph (font_info,
 		    	shiftdown_tone_ad (ucs2tis (cluster[1]), shaping_table));
 	    return 2;
 	  }
@@ -396,8 +396,8 @@ get_adjusted_glyphs_list (ThaiFontInfo *font_info,
           	 is_char_type (cluster[1], AboveVowel))
 	  {
 	    glyph_lists[0] =
-		(*font_info->make_glyph) (font_info, ucs2tis (cluster[0]));
-	    glyph_lists[1] = (*font_info->make_glyph) (font_info,
+		thai_make_glyph (font_info, ucs2tis (cluster[0]));
+	    glyph_lists[1] = thai_make_glyph (font_info,
 		    	shiftleft_av (ucs2tis (cluster[1]), shaping_table));
 	    return 2;
           }
@@ -405,8 +405,8 @@ get_adjusted_glyphs_list (ThaiFontInfo *font_info,
           	 is_char_type (cluster[1], AboveDiac|Tone))
 	  {
 	    glyph_lists[0] =
-		(*font_info->make_glyph) (font_info, ucs2tis (cluster[0]));
-	    glyph_lists[1] = (*font_info->make_glyph) (font_info,
+		thai_make_glyph (font_info, ucs2tis (cluster[0]));
+	    glyph_lists[1] = thai_make_glyph (font_info,
 			shiftdownleft_tone_ad (ucs2tis (cluster[1]), shaping_table));
 	    return 2;
 	  }
@@ -414,40 +414,40 @@ get_adjusted_glyphs_list (ThaiFontInfo *font_info,
           	 is_char_type (cluster[1], BelowVowel|BelowDiac))
 	  {
 	    glyph_lists[0] =
-		(*font_info->make_glyph) (font_info, ucs2tis (cluster[0]));
+		thai_make_glyph (font_info, ucs2tis (cluster[0]));
 	    glyph_lists[1] =
-		(*font_info->make_glyph) (font_info, ucs2tis (cluster[1]));
+		thai_make_glyph (font_info, ucs2tis (cluster[1]));
 	    return 2;
           }
 	else if (is_char_type (cluster[0], BotTailCons) &&
 		 is_char_type (cluster[1], BelowVowel|BelowDiac))
 	  {
 	    glyph_lists[0] =
-		(*font_info->make_glyph) (font_info, ucs2tis (cluster[0]));
+		thai_make_glyph (font_info, ucs2tis (cluster[0]));
 	    glyph_lists[1] =
-		(*font_info->make_glyph) (font_info,
+		thai_make_glyph (font_info,
 			shiftdown_bv_bd (ucs2tis (cluster[1]), shaping_table));
 	    return 2;
 	  }
 	else if (is_char_type (cluster[0], SpltTailCons) &&
           	 is_char_type (cluster[1], BelowVowel|BelowDiac))
 	  {
-	    glyph_lists[0] = (*font_info->make_glyph) (font_info,
+	    glyph_lists[0] = thai_make_glyph (font_info,
 		    		tailcutcons (ucs2tis (cluster[0]), shaping_table));
 	    glyph_lists[1] =
-		(*font_info->make_glyph) (font_info, ucs2tis (cluster[1]));
+		thai_make_glyph (font_info, ucs2tis (cluster[1]));
 	    return 2;
 	  }
 	else
 	  {
 	    if (font_info->font_set == THAI_FONT_TIS)
-	      glyph_lists[0] = (*font_info->make_glyph) (font_info, 0x20);
+	      glyph_lists[0] = thai_make_glyph (font_info, 0x20);
 	    else
-	      glyph_lists[0] = (*font_info->make_glyph) (font_info, 0x7F);
+	      glyph_lists[0] = thai_make_glyph (font_info, 0x7F);
 	    glyph_lists[1] =
-		(*font_info->make_glyph) (font_info, ucs2tis (cluster[0]));
+		thai_make_glyph (font_info, ucs2tis (cluster[0]));
 	    glyph_lists[2] =
-		(*font_info->make_glyph) (font_info, ucs2tis (cluster[1]));
+		thai_make_glyph (font_info, ucs2tis (cluster[1]));
 	    return 3;
 	  }
         break;
@@ -458,11 +458,11 @@ get_adjusted_glyphs_list (ThaiFontInfo *font_info,
             is_char_type (cluster[2], SaraAm))
 	  {
             glyph_lists[0] =
-		(*font_info->make_glyph) (font_info, ucs2tis (cluster[0]));
-            glyph_lists[1] = (*font_info->make_glyph) (font_info, 0xED);
+		thai_make_glyph (font_info, ucs2tis (cluster[0]));
+            glyph_lists[1] = thai_make_glyph (font_info, 0xED);
             glyph_lists[2] =
-		(*font_info->make_glyph) (font_info, ucs2tis (cluster[1]));
-            glyph_lists[3] = (*font_info->make_glyph) (font_info, 0xD2);
+		thai_make_glyph (font_info, ucs2tis (cluster[1]));
+            glyph_lists[3] = thai_make_glyph (font_info, 0xD2);
             return 4;
           }
 	else if (is_char_type (cluster[0], UpTailCons) &&
@@ -470,12 +470,12 @@ get_adjusted_glyphs_list (ThaiFontInfo *font_info,
 		 is_char_type (cluster[2], SaraAm))
 	  {
 	    glyph_lists[0] =
-		(*font_info->make_glyph) (font_info, ucs2tis (cluster[0]));
-	    glyph_lists[1] = (*font_info->make_glyph) (font_info,
+		thai_make_glyph (font_info, ucs2tis (cluster[0]));
+	    glyph_lists[1] = thai_make_glyph (font_info,
 				shiftleft_tone_ad (0xED, shaping_table));
-	    glyph_lists[2] = (*font_info->make_glyph) (font_info,
+	    glyph_lists[2] = thai_make_glyph (font_info,
 		    		shiftleft_tone_ad (ucs2tis (cluster[1]), shaping_table));
-	    glyph_lists[3] = (*font_info->make_glyph) (font_info, 0xD2);
+	    glyph_lists[3] = thai_make_glyph (font_info, 0xD2);
 	    return 4;
 	  }
 	else if (is_char_type (cluster[0], UpTailCons) &&
@@ -483,10 +483,10 @@ get_adjusted_glyphs_list (ThaiFontInfo *font_info,
 		 is_char_type (cluster[2], AboveDiac|Tone))
 	  {
 	    glyph_lists[0] =
-		(*font_info->make_glyph) (font_info, ucs2tis (cluster[0]));
-	    glyph_lists[1] = (*font_info->make_glyph) (font_info,
+		thai_make_glyph (font_info, ucs2tis (cluster[0]));
+	    glyph_lists[1] = thai_make_glyph (font_info,
 				shiftleft_av (ucs2tis (cluster[1]), shaping_table));
-	    glyph_lists[2] = (*font_info->make_glyph) (font_info,
+	    glyph_lists[2] = thai_make_glyph (font_info,
 		    		shiftleft_tone_ad (ucs2tis (cluster[2]), shaping_table));
 	    return 3;
 	  }
@@ -495,10 +495,10 @@ get_adjusted_glyphs_list (ThaiFontInfo *font_info,
 		 is_char_type (cluster[2], AboveDiac|Tone))
 	  {
 	    glyph_lists[0] =
-		(*font_info->make_glyph) (font_info, ucs2tis (cluster[0]));
+		thai_make_glyph (font_info, ucs2tis (cluster[0]));
 	    glyph_lists[1] =
-		(*font_info->make_glyph) (font_info, ucs2tis (cluster[1]));
-	    glyph_lists[2] = (*font_info->make_glyph) (font_info,
+		thai_make_glyph (font_info, ucs2tis (cluster[1]));
+	    glyph_lists[2] = thai_make_glyph (font_info,
 			shiftdownleft_tone_ad (ucs2tis (cluster[2]), shaping_table));
 	    return 3;
 	  }
@@ -507,11 +507,11 @@ get_adjusted_glyphs_list (ThaiFontInfo *font_info,
 		 is_char_type (cluster[2], AboveDiac|Tone))
 	  {
 	    glyph_lists[0] =
-		(*font_info->make_glyph) (font_info, ucs2tis (cluster[0]));
+		thai_make_glyph (font_info, ucs2tis (cluster[0]));
 	    glyph_lists[1] =
-		(*font_info->make_glyph) (font_info, ucs2tis (cluster[1]));
+		thai_make_glyph (font_info, ucs2tis (cluster[1]));
 	    glyph_lists[2] =
-		(*font_info->make_glyph) (font_info,
+		thai_make_glyph (font_info,
 			shiftdown_tone_ad (ucs2tis (cluster[2]), shaping_table));
 	    return 3;
 	  }
@@ -519,11 +519,11 @@ get_adjusted_glyphs_list (ThaiFontInfo *font_info,
 		 is_char_type (cluster[1], BelowVowel) &&
 		 is_char_type (cluster[2], AboveDiac|Tone))
 	  {
-	    glyph_lists[0] = (*font_info->make_glyph) (font_info,
+	    glyph_lists[0] = thai_make_glyph (font_info,
 		    		tailcutcons (ucs2tis (cluster[0]), shaping_table));
 	    glyph_lists[1] =
-		(*font_info->make_glyph) (font_info, ucs2tis (cluster[1]));
-	    glyph_lists[2] = (*font_info->make_glyph) (font_info,
+		thai_make_glyph (font_info, ucs2tis (cluster[1]));
+	    glyph_lists[2] = thai_make_glyph (font_info,
 		    		shiftdown_tone_ad (ucs2tis (cluster[2]), shaping_table));
 	    return 3;
 	  }
@@ -532,21 +532,21 @@ get_adjusted_glyphs_list (ThaiFontInfo *font_info,
 		 is_char_type (cluster[2], AboveDiac|Tone))
 	  {
 	    glyph_lists[0] =
-		(*font_info->make_glyph) (font_info, ucs2tis (cluster[0]));
-	    glyph_lists[1] = (*font_info->make_glyph) (font_info,
+		thai_make_glyph (font_info, ucs2tis (cluster[0]));
+	    glyph_lists[1] = thai_make_glyph (font_info,
 		    		shiftdown_bv_bd (ucs2tis (cluster[1]), shaping_table));
-	    glyph_lists[2] = (*font_info->make_glyph) (font_info,
+	    glyph_lists[2] = thai_make_glyph (font_info,
 				shiftdown_tone_ad (ucs2tis (cluster[2]), shaping_table));
 	    return 3;
 	  }
 	else
 	  {
 	    glyph_lists[0] =
-		(*font_info->make_glyph) (font_info, ucs2tis (cluster[0]));
+		thai_make_glyph (font_info, ucs2tis (cluster[0]));
 	    glyph_lists[1] =
-		(*font_info->make_glyph) (font_info, ucs2tis (cluster[1]));
+		thai_make_glyph (font_info, ucs2tis (cluster[1]));
 	    glyph_lists[2] =
-		(*font_info->make_glyph) (font_info, ucs2tis (cluster[2]));
+		thai_make_glyph (font_info, ucs2tis (cluster[2]));
 	    return 3;
 	  }
       break;
@@ -561,39 +561,15 @@ get_glyphs_list (ThaiFontInfo	*font_info,
 		 gint		num_chrs,
 		 PangoGlyph	*glyph_lists)
 {
-  PangoGlyph glyph;
-  gint xtis_index;
   gint i;
 
   switch (font_info->font_set)
     {
       case THAI_FONT_NONE:
         for (i=0; i < num_chrs; i++)
-	  glyph_lists[i] = (*font_info->make_unknown_glyph) (font_info, glyph_lists[i]);
+	  glyph_lists[i] = thai_make_unknown_glyph (font_info, glyph_lists[i]);
         return num_chrs;
 
-#ifdef HAVE_X
-      case THAI_FONT_XTIS:
-        /* If we are rendering with an XTIS font, we try to find a precomposed
-         * glyph for the cluster.
-         */
-        xtis_index = 0x100 * (cluster[0] - 0xe00 + 0x20) + 0x30;
-        if (cluster[1])
-	    xtis_index +=8 * group1_map[cluster[1] - 0xe30];
-        if (cluster[2])
-	    xtis_index += group2_map[cluster[2] - 0xe30];
-        glyph = (*font_info->make_glyph) (font_info, xtis_index);
-	if ((*font_info->has_glyph) (font_info, glyph)) {
-            glyph_lists[0] = glyph;
-            return 1;
-        }
-        for (i=0; i < num_chrs; i++)
-	  glyph_lists[i] =
-	      (*font_info->make_glyph) (font_info,
-			0x100 * (cluster[i] - 0xe00 + 0x20) + 0x30);
-        return num_chrs;
-#endif
-      
       case THAI_FONT_TIS:
 	/* TIS620-0 + Wtt2.0 Extension
 	 */
@@ -614,7 +590,7 @@ get_glyphs_list (ThaiFontInfo	*font_info,
       
       case THAI_FONT_ISO10646:
         for (i=0; i < num_chrs; i++)
-          glyph_lists[i] = (*font_info->make_glyph) (font_info, cluster[i]);
+          glyph_lists[i] = thai_make_glyph (font_info, cluster[i]);
         return num_chrs;
     }
 
@@ -631,11 +607,21 @@ add_cluster (ThaiFontInfo	*font_info,
   PangoGlyph glyphs_list[MAX_GLYPHS];
   gint num_glyphs;
   gint i;
-  
-  num_glyphs = get_glyphs_list(font_info, cluster, num_chrs, glyphs_list);
-  for (i=0; i<num_glyphs; i++)
-       add_glyph (font_info, glyphs, cluster_start, glyphs_list[i],
-	    		i == 0 ? FALSE : TRUE);
+
+  if (!isthai (cluster[0]))
+    {
+      g_assert (num_chrs == 1);
+      add_glyph (font_info, glyphs, cluster_start,
+		 thai_make_glyph (font_info, cluster[0]),
+		 FALSE);
+    }
+  else
+    {
+      num_glyphs = get_glyphs_list(font_info, cluster, num_chrs, glyphs_list);
+      for (i=0; i<num_glyphs; i++)
+	add_glyph (font_info, glyphs, cluster_start, glyphs_list[i],
+		   i == 0 ? FALSE : TRUE);
+    }
 }
 
 static gboolean
@@ -671,16 +657,26 @@ get_next_cluster(const char	*text,
   while (p < text + length && n_chars < 3)  
     {
       gunichar current = g_utf8_get_char (p);
-      
-      if (n_chars == 0 ||
-	  is_wtt_composible ((gunichar)(cluster[n_chars - 1]), current) ||
-	  (n_chars == 1 &&
-	   is_char_type (cluster[0], Cons) && 
-	   is_char_type (current, SaraAm)) ||
-	  (n_chars == 2 &&
-	   is_char_type (cluster[0], Cons) &&
-	   is_char_type (cluster[1], Tone) &&
-	   is_char_type (current, SaraAm)))
+
+      /* Non-thai characters get split into a single character cluster */
+      if (!isthai (current))
+	{
+	  if (n_chars == 0)
+	    {
+	      cluster[n_chars++] = current;
+	      p = g_utf8_next_char (p);
+	    }
+	  break;
+	}
+      else if (n_chars == 0 ||
+	       is_wtt_composible ((gunichar)(cluster[n_chars - 1]), current) ||
+	       (n_chars == 1 &&
+		is_char_type (cluster[0], Cons) && 
+		is_char_type (current, SaraAm)) ||
+	       (n_chars == 2 &&
+		is_char_type (cluster[0], Cons) &&
+		is_char_type (cluster[1], Tone) &&
+		is_char_type (current, SaraAm)))
 	{
 	  cluster[n_chars++] = current;
 	  p = g_utf8_next_char (p);
@@ -694,12 +690,14 @@ get_next_cluster(const char	*text,
 }
 
 void 
-thai_engine_shape (ThaiFontInfo     *font_info,
+thai_engine_shape (PangoEngineShape *engine,
+		   PangoFont        *font,
 		   const char       *text,
 		   gint              length,
 		   PangoAnalysis    *analysis,
 		   PangoGlyphString *glyphs)
 {
+  ThaiFontInfo *font_info;
   const char *p;
   const char *log_cluster;
   gunichar cluster[MAX_CLUSTER_CHRS];
@@ -707,6 +705,8 @@ thai_engine_shape (ThaiFontInfo     *font_info,
 
   pango_glyph_string_set_size (glyphs, 0);
 
+  font_info = thai_get_font_info (font);
+
   p = text;
   while (p < text + length)
     {
